////////////////////////////////////////////////////////////////////////////////////////////
// LEDMulti.c - plik gwny programu obsugi multipleksowanego wywietlacza LED
//     przeznaczenie: "Programowanie procesorw w jzyku C (EdW)"
//
// Autor: Radosaw Koppel          Kompilator: WinAVR 20050214
////////////////////////////////////////////////////////////////////////////////////////////
// Komenta dodatkowy: 
//   Plik napisany w celu pokazania zasigu zmiennych lokalnych
//
/////////////////////////////////////////////////////////////////////////////////////////////
#include <avr\io.h>
#include <inttypes.h>
#include <avr\signal.h>
#include <avr\interrupt.h>
#include <avr\pgmspace.h>

// Definicje wyprowadze
#define LED_A 0
#define LED_B 1
#define LED_C 2
#define LED_D 3
#define LED_E 4
#define LED_F 5
#define LED_G 6
#define LED_DP 7
#define LEDPORT PORTB
#define LEDDDR DDRB
#define COM1 6
#define COM2 5
#define COM3 4
#define COM4 3
#define COMPORT PORTD
#define COMDDR DDRD
#define SWPORT PORTD
#define SWDDR DDRD
#define SWPIN PIND
#define SW1 0
#define SW2 1

volatile uint8_t g_bDelay; // 122Hz
uint8_t g_DaneWyswietlacza[4]; 
prog_uint8_t g_DaneCom[4] = 
	{~(1<<COM1), ~(1<<COM2), ~(1<<COM3), ~(1<<COM4)}; 

prog_uint8_t g_WzorCyfr[16] = 
	{
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F), // 0	
		~(1<<LED_B | 1<<LED_C), // 1
		~(1<<LED_A | 1<<LED_B | 1<<LED_G | 1<<LED_E | 1<<LED_D), // 2
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_G), // 3
		~(1<<LED_B | 1<<LED_C | 1<<LED_G | 1<<LED_F), // 4
		~(1<<LED_A | 1<<LED_F | 1<<LED_G | 1<<LED_C | 1<<LED_D), // 5
		~(1<<LED_A | 1<<LED_F | 1<<LED_E | 1<<LED_D | 1<<LED_C |1<< LED_G), // 6
		~(1<<LED_A | 1<<LED_B | 1<<LED_C), // 7
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // 8
		~(1<<LED_G | 1<<LED_F | 1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D), // 9
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_E | 1<<LED_F | 1<<LED_G), // A
		~(1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // B
		~(1<<LED_A | 1<<LED_D | 1<<LED_E | 1<<LED_F), // C
		~(1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_G), // D
		~(1<<LED_A | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // E
		~(1<<LED_A | 1<<LED_E | 1<<LED_F | 1<<LED_G) // F
	}; 

uint16_t g_Licznik = 0; 

// Funkcja wywietlajca dan liczb dziesitnie
void WyswietlDEC(uint16_t var)
{
	uint8_t n; 
	if(var > 9999)
	{
		for(n=0; n<4; n++)
		{
			g_DaneWyswietlacza[n] = ~(1<<LED_G); 
		}
		return; 
	}
	for(n=4; n>0; n--)
	{
		g_DaneWyswietlacza[n-1] = pgm_read_byte(&g_WzorCyfr[var%10]); 
		var /= 10; 
	}
}

// Start
int main(void)
{
	/////////////////////////////
	// inicjacja
	LEDDDR = 0xff; 
	COMDDR = 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	SWPORT = 1<<SW1 | 1<<SW2; // wczenie podcigania na wyprowadzeniach przyciskw
	// Timer0
	TCCR0 = 1<<CS01|1<<CS00; 
	TIMSK = 1<<TOIE0; 
	// Globalne zezwolenie na przerwania
	sei(); 
	// koniec inicjacji
	/////////////////////////////

	for(;;)
	{
		uint8_t a = 1; 
		WyswietlDEC(a); 
		if(!(SWPIN & 1<<SW1))
		{
			// usuwanie drga stykw
			g_bDelay = 6; //50ms
			while(g_bDelay != 0) {}
			if(!(SWPIN & 1<<SW1))
			{
				uint8_t a = 100; 
				--a; 
				WyswietlDEC(a); 
				// Oczekiwanie na puszczenie
				while(!(SWPIN & 1<<SW1))
				{
				}
			}
		}
		if(!(SWPIN & 1<<SW2))
		{
			// usuwanie drga stykw
			g_bDelay = 6; //50ms
			while(g_bDelay != 0) {}
			if(!(SWPIN & 1<<SW2))
			{
				++a; 
				g_bDelay = 120; //1s
				while(!(SWPIN & 1<<SW2))
				{
				}
			}
		}
	}

	return 0; 
}

//_________________________________________________________________________________
// Obsuga przerwa
SIGNAL(SIG_OVERFLOW0)
{
	static uint8_t AktWyswietlacz = 0; 
	// Wpisanie do licznika pocztkowej wartoci
	TCNT0 = 128; 
	// Wygaszenie wywietlaczy
	COMPORT |= 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	// Wysanie odpowiedniej danej
	LEDPORT = g_DaneWyswietlacza[AktWyswietlacz]; 
	// Wczenie odpowiedniego wywietlacza
	COMPORT &= pgm_read_byte(&g_DaneCom[AktWyswietlacz]); 
	// Zwikszenie stanu zmiennej wskazujcej na obsugiwany wywietlacz
	++AktWyswietlacz; 
	if(AktWyswietlacz > 3)
	{
		// Zmienna zapobiegajca niepotrzebnemu kopiowaniu zmiennej g_bDelay (typ volatile!)
		uint8_t delay; 
		AktWyswietlacz = 0; 
		// Obsuga opnie
		delay = g_bDelay; 
		if(delay > 0)
		{
			--delay; 
			g_bDelay = delay; 
		}
	}
}
